var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Alert, Button, Center, Input, Spinner, VStack } from "@chakra-ui/react";
import { yupResolver } from "@hookform/resolvers/yup";
import { DataV2, Metadata } from "@metaplex-foundation/mpl-token-metadata";
import { useWallet } from "@solana/wallet-adapter-react";
import { PublicKey } from "@solana/web3.js";
import { MarketplaceSdk } from "@strata-foundation/marketplace-sdk";
import { truthy, useProvider, useStrataSdks, useTokenBondingFromMint, useTokenMetadata } from "@strata-foundation/react";
import { SplTokenBonding } from "@strata-foundation/spl-token-bonding";
import { SplTokenMetadata } from "@strata-foundation/spl-utils";
import { useRouter } from "next/router";
import React from "react";
import { useAsync, useAsyncCallback } from "react-async-hook";
import { FormProvider, useForm } from "react-hook-form";
import { route, routes } from "../../utils/routes";
import * as yup from "yup";
import { FormControlWithError } from "./FormControlWithError";
import { Recipient } from "./Recipient";
import { TokenMetadataInputs } from "./TokenMetadataInputs";
const validationSchema = yup.object({
    image: yup.mixed(),
    name: yup.string().required().min(2),
    description: yup.string(),
    shortName: yup.string().min(2).max(10),
    contact: yup.string(),
    discussion: yup.string(),
    authority: yup.string().required(),
});
function editBounty(tokenMetadataSdk, tokenBondingSdk, values, mintKey) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const uri = yield tokenMetadataSdk.createArweaveMetadata({
            name: values.name,
            symbol: values.shortName,
            description: values.description,
            image: (_a = values.image) === null || _a === void 0 ? void 0 : _a.name,
            files: [values.image].filter(truthy),
            mint: mintKey,
            attributes: MarketplaceSdk.bountyAttributes({
                mint: mintKey,
                discussion: values.discussion,
                contact: values.contact,
            }),
        });
        const instructions = [];
        const signers = [];
        const { instructions: metaInstrs, signers: metaSigners } = yield tokenMetadataSdk.updateMetadataInstructions({
            metadata: yield Metadata.getPDA(mintKey),
            data: new DataV2({
                // Max name len 32
                name: values.name.substring(0, 32),
                symbol: values.shortName.substring(0, 10),
                uri,
                sellerFeeBasisPoints: 0,
                creators: null,
                collection: null,
                uses: null,
            }),
        });
        instructions.push(...metaInstrs);
        signers.push(...metaSigners);
        const tokenBondingKey = (yield SplTokenBonding.tokenBondingKey(mintKey))[0];
        const tokenBondingAcct = yield tokenBondingSdk.getTokenBonding(tokenBondingKey);
        if (values.authority && tokenBondingSdk.wallet.publicKey.equals(tokenBondingAcct.reserveAuthority)) {
            const authority = new PublicKey(values.authority);
            const { instructions: bondInstrs, signers: bondSigners } = yield tokenBondingSdk.updateTokenBondingInstructions({
                tokenBonding: tokenBondingKey,
                generalAuthority: authority,
                reserveAuthority: authority,
            });
            instructions.push(...bondInstrs);
            signers.push(...bondSigners);
        }
        yield tokenBondingSdk.sendInstructions(instructions, signers);
        return mintKey;
    });
}
export const EditBountyFormRaw = ({ mintKey, values, }) => {
    const formProps = useForm({
        resolver: yupResolver(validationSchema),
        defaultValues: values,
    });
    const { register, handleSubmit, watch, setValue, formState: { errors, isSubmitting }, } = formProps;
    const { publicKey } = useWallet();
    const { awaitingApproval } = useProvider();
    const { execute, loading, error } = useAsyncCallback(editBounty);
    const { tokenMetadataSdk, tokenBondingSdk } = useStrataSdks();
    const router = useRouter();
    const authority = watch("authority");
    const onSubmit = (values) => __awaiter(void 0, void 0, void 0, function* () {
        yield execute(tokenMetadataSdk, tokenBondingSdk, values, mintKey);
        router.push(route(routes.bounty, { mintKey: mintKey.toBase58() }));
    });
    const authorityRegister = register("authority");
    return (<FormProvider {...formProps}>
      <form onSubmit={handleSubmit(onSubmit)}>
        <VStack spacing={8}>
          <TokenMetadataInputs />
          <FormControlWithError id="shortName" help="A less than 10 character name for this bounty. This will be the bounty token's symbol." label="Short Name" errors={errors}>
            <Input {...register("shortName")}/>
          </FormControlWithError>

          <FormControlWithError id="authority" help="The wallet that signs to disburse the funds of this bounty when it is completed. 
            For social tokens, this defaults to the wallet associated with the social token. This
            can also be an SPL Governance address or a multisig." label="Approver" errors={errors}>
            {publicKey && (<Button variant="link" onClick={() => setValue("authority", publicKey.toBase58())}>
                Set to My Wallet
              </Button>)}
            <Recipient name={authorityRegister.name} value={authority} onChange={authorityRegister.onChange}/>
          </FormControlWithError>
          <FormControlWithError id="contact" help="Who to contact regarding the bounty. This can be an email address, twitter handle, etc." label="Contact Information" errors={errors}>
            <Input {...register("contact")}/>
          </FormControlWithError>
          <FormControlWithError id="discussion" help="A link to where this bounty is actively being discussed. This can be a github issue, forum link, etc. Use this to coordinate the bounty." label="Discussion" errors={errors}>
            <Input {...register("discussion")}/>
          </FormControlWithError>

          {error && <Alert status="error">{error.toString()}</Alert>}

          <Button type="submit" alignSelf="flex-end" colorScheme="orange" isLoading={isSubmitting || loading} loadingText={awaitingApproval ? "Awaiting Approval" : "Loading"}>
            Send Bounty
          </Button>
        </VStack>
      </form>
    </FormProvider>);
};
const getFileFromUrl = (url, name, defaultType = "image/jpeg") => __awaiter(void 0, void 0, void 0, function* () {
    if (!url) {
        return undefined;
    }
    const data = yield fetch(url, { cache: "no-cache" });
    const blob = yield data.blob();
    const fileName = `${name}${blob.type === defaultType ? ".jpeg" : "png"}`;
    const file = new File([blob], fileName, { type: blob.type || defaultType });
    return file;
});
export const EditBountyForm = ({ mintKey }) => {
    const { info: tokenBonding, loading: loadingBonding } = useTokenBondingFromMint(mintKey);
    const { displayName, image, metadata, data, loading: loadingMetadata, } = useTokenMetadata(mintKey);
    const { result: file, loading } = useAsync(getFileFromUrl, [image, "image"]);
    const attributes = React.useMemo(() => SplTokenMetadata.attributesToRecord(data === null || data === void 0 ? void 0 : data.attributes), [data]);
    if (loadingBonding ||
        loadingMetadata ||
        loading ||
        !tokenBonding ||
        !metadata) {
        return (<Center>
        <Spinner />
      </Center>);
    }
    return (<EditBountyFormRaw mintKey={mintKey} values={{
            authority: tokenBonding.reserveAuthority.toBase58(),
            name: displayName,
            image: file,
            description: data.description,
            shortName: metadata.data.symbol,
            contact: attributes.contact,
            discussion: attributes.discussion,
        }}/>);
};
